package cn.com.qmth.examcloud.web.interceptor;

import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.servlet.HandlerInterceptor;

import com.codahale.metrics.ConsoleReporter;
import com.codahale.metrics.Meter;
import com.codahale.metrics.MetricRegistry;
import com.codahale.metrics.Timer;
import com.codahale.metrics.Timer.Context;

import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import cn.com.qmth.examcloud.web.support.ApiInfo;

/**
 * 统计拦截器
 *
 * @author WANGWEI
 * @date 2019年7月24日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class StatisticInterceptor implements HandlerInterceptor {

	private MetricRegistry metricRegistry = new MetricRegistry();

	ConsoleReporter reporter = ConsoleReporter.forRegistry(metricRegistry)
			.convertRatesTo(TimeUnit.SECONDS).convertDurationsTo(TimeUnit.MILLISECONDS).build();

	public StatisticInterceptor() {
		reporter.start(1, TimeUnit.SECONDS);// 1s报道一次
	}

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {

		ApiInfo apiInfo = (ApiInfo) request
				.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

		if (null == apiInfo) {
			return true;
		}

		Timer timer = metricRegistry
				.timer(MetricRegistry.name(apiInfo.getMapping(), apiInfo.getDescription()));
		Context ctx = timer.time();
		request.setAttribute(HttpServletRequestAttribute.$_METRICS_TIMER_CTX.name(), ctx);

		return true;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {
		Context ctx = (Context) request
				.getAttribute(HttpServletRequestAttribute.$_METRICS_TIMER_CTX.name());
		if (null != ctx) {
			ctx.stop();
		}
		Boolean hasException = (Boolean) request
				.getAttribute(HttpServletRequestAttribute.$_EXCEPTION_HAPPENED.name());

		if (null != hasException && hasException) {
			ApiInfo apiInfo = (ApiInfo) request
					.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

			if (null != apiInfo) {
				Meter meter = metricRegistry.meter(apiInfo.getMapping());
				meter.mark();
			}
		}

	}
}
